@@ -121,11 +121,10 @@ string getPathOfIndexList(const Config& config, const IndexEntry& entry)
 	return basePath + "_" + indexListSuffix;
 }
 
-vector< Cache::IndexDownloadRecord > getDownloadInfoOfIndexList(
-		const Config& config, const IndexEntry& indexEntry)
+static vector< Cache::IndexDownloadRecord > getDownloadInfoFromRelease(
+		const Config& config, const IndexEntry& indexEntry, const string& suffix)
 {
 	auto baseUri = getUriOfIndexEntry(indexEntry);
-	auto indexListSuffix = getIndexListSuffix(config, indexEntry, '/');
 
 	vector< Cache::IndexDownloadRecord > result;
 	{ // reading
@@ -134,8 +133,7 @@ vector< Cache::IndexDownloadRecord > getDownloadInfoOfIndexList(
 		File releaseFile(releaseFilePath, "r", openError);
 		if (!openError.empty())
 		{
-			fatal("unable to open release file '%s': %s",
-					releaseFilePath.c_str(), openError.c_str());
+			fatal2("unable to open release file '%s': %s", releaseFilePath, openError);
 		}
 
 		HashSums::Type currentHashSumType = HashSums::Count;
@@ -156,24 +154,23 @@ vector< Cache::IndexDownloadRecord > getDownloadInfoOfIndexList(
 			{
 				currentHashSumType = HashSums::SHA256;
 			}
-			else if (line.find(indexListSuffix) != string::npos)
+			else if (line.find(suffix) != string::npos)
 			{
 				if (currentHashSumType == HashSums::Count)
 				{
-					fatal("release line '%s' without previous hash sum declaration at release file '%s'",
-								line.c_str(), releaseFilePath.c_str());
+					fatal2("release line '%s' without previous hash sum declaration at release file '%s'",
+								line, releaseFilePath);
 				}
 				static sregex hashSumsLineRegex = sregex::compile("\\s([[:xdigit:]]+)\\s+(\\d+)\\s+(.*)");
 				if (!regex_match(line, m, hashSumsLineRegex))
 				{
-					fatal("malformed release line '%s' at file '%s'",
-								line.c_str(), releaseFilePath.c_str());
+					fatal2("malformed release line '%s' at file '%s'", line, releaseFilePath);
 				}
 
 				string name = m[3];
-				if (name.compare(0, indexListSuffix.size(), indexListSuffix) != 0)
+				if (name.compare(0, suffix.size(), suffix) != 0)
 				{
-					continue; // doesn't start with indexListSuffix
+					continue; // doesn't start with suffix
 				}
 
 				// filling result structure
@@ -205,13 +202,20 @@ vector< Cache::IndexDownloadRecord > getDownloadInfoOfIndexList(
 	{
 		if (recordIt->hashSums.empty())
 		{
-			fatal("no hash sums defined for index list URI '%s'", recordIt->uri.c_str());
+			fatal2("no hash sums defined for index list URI '%s'", recordIt->uri);
 		}
 	}
 
 	return result;
 }
 
+vector< Cache::IndexDownloadRecord > getDownloadInfoOfIndexList(
+		const Config& config, const IndexEntry& indexEntry)
+{
+	return getDownloadInfoFromRelease(config, indexEntry,
+			getIndexListSuffix(config, indexEntry, '/'));
+}
+
 static vector< vector< string > > getChunksOfLocalizedDescriptions(
 		const Config& config, const IndexEntry& entry)
 {
@@ -222,38 +226,51 @@ static vector< vector< string > > getChunksOfLocalizedDescriptions(
 		return result;
 	}
 
-	auto translationVariable = config.getString("apt::acquire::translation");
-	auto locale = translationVariable == "environment" ?
-			setlocale(LC_MESSAGES, NULL) : translationVariable;
-	if (locale == "none")
-	{
-		return result;
-	}
-
-	vector< string > chunks;
+	vector< string > chunksBase;
 	if (!entry.component.empty())
 	{
-		chunks.push_back(entry.component);
+		chunksBase.push_back(entry.component);
 	}
-	chunks.push_back("i18n");
+	chunksBase.push_back("i18n");
 
-	result.push_back(chunks);
-	// cutting out an encoding
-	auto dotPosition = locale.rfind('.');
-	if (dotPosition != string::npos)
+	set< string > alreadyAddedTranslations;
+	auto addTranslation = [&chunksBase, &alreadyAddedTranslations, &result](const string& locale)
 	{
-		locale.erase(dotPosition);
-	}
-	result[0].push_back(string("Translation-") + locale);
+		if (alreadyAddedTranslations.insert(locale).second)
+		{
+			auto chunks = chunksBase;
+			chunks.push_back(string("Translation-") + locale);
+			result.push_back(chunks);
+		}
+	};
 
-	result.push_back(chunks);
-	// cutting out an country specificator
-	auto underlinePosition = locale.rfind('_');
-	if (underlinePosition != string::npos)
+	auto translationVariable = config.getString("cupt::languages::indexes");
+	auto translations = split(',', translationVariable);
+	FORIT(translationIt, translations)
 	{
-		locale.erase(underlinePosition);
+		auto locale = (*translationIt == "environment") ?
+				setlocale(LC_MESSAGES, NULL) : *translationIt;
+		if (locale == "none")
+		{
+			continue;
+		}
+
+		// cutting out an encoding
+		auto dotPosition = locale.rfind('.');
+		if (dotPosition != string::npos)
+		{
+			locale.erase(dotPosition);
+		}
+		addTranslation(locale);
+
+		// cutting out an country specificator
+		auto underlinePosition = locale.rfind('_');
+		if (underlinePosition != string::npos)
+		{
+			locale.erase(underlinePosition);
+		}
+		addTranslation(locale);
 	}
-	result[1].push_back(string("Translation-") + locale);
 
 	return result;
 }
@@ -293,6 +310,31 @@ vector< Cache::LocalizationDownloadRecord > getDownloadInfoOfLocalizedDescriptio
 	return result;
 }
 
+vector< FileDownloadRecord > getDownloadInfoOfLocalizationIndex(const Config& config,
+		const IndexEntry& entry)
+{
+	return getDownloadInfoFromRelease(config, entry, entry.component + "/i18n/Index");
+}
+
+vector< LocalizationDownloadRecord2 > getDownloadInfoOfLocalizedDescriptions2(
+		const Config& config, const IndexEntry& entry)
+{
+	auto chunkArrays = getChunksOfLocalizedDescriptions(config, entry);
+	auto basePath = getPathOfIndexEntry(config, entry);
+
+	vector< LocalizationDownloadRecord2 > result;
+
+	FORIT(chunkArrayIt, chunkArrays)
+	{
+		LocalizationDownloadRecord2 record;
+		record.localPath = basePath + "_" + join("_", *chunkArrayIt);
+		record.filePart = *(chunkArrayIt->rbegin()); // i.e. 'Translation-xyz' part
+		result.push_back(std::move(record));
+	}
+
+	return result;
+}
+
 string getPathOfExtendedStates(const Config& config)
 {
 	return config.getPath("dir::state::extendedstates");
@@ -303,26 +345,26 @@ bool verifySignature(const Config& config, const string& path)
 	auto debugging = config.getBool("debug::gpgv");
 	if (debugging)
 	{
-		debug("verifying file '%s'", path.c_str());
+		debug2("verifying file '%s'", path);
 	}
 
 	auto keyringPath = config.getString("gpgv::trustedkeyring");
 	if (debugging)
 	{
-		debug("keyring file is '%s'", keyringPath.c_str());
+		debug2("keyring file is '%s'", keyringPath);
 	}
 
 	auto signaturePath = path + ".gpg";
 	if (debugging)
 	{
-		debug("signature file is '%s'", signaturePath.c_str());
+		debug2("signature file is '%s'", signaturePath);
 	}
 
 	if (!fs::fileExists(signaturePath))
 	{
 		if (debugging)
 		{
-			debug("signature file '%s' doesn't exist", signaturePath.c_str());
+			debug2("signature file '%s' doesn't exist", signaturePath);
 		}
 		return 0;
 	}
@@ -335,8 +377,7 @@ bool verifySignature(const Config& config, const string& path)
 		{
 			if (debugging)
 			{
-				debug("unable to read signature file '%s': %s",
-						signaturePath.c_str(), openError.c_str());
+				debug2("unable to read signature file '%s': %s", signaturePath, openError);
 			}
 			return false;
 		}
@@ -348,8 +389,7 @@ bool verifySignature(const Config& config, const string& path)
 		{
 			if (debugging)
 			{
-				debug("unable to read keyring file '%s': %s",
-						keyringPath.c_str(), openError.c_str());
+				debug2("unable to read keyring file '%s': %s", keyringPath, openError);
 			}
 			return false;
 		}
@@ -364,7 +404,7 @@ bool verifySignature(const Config& config, const string& path)
 		File gpgPipe(gpgCommand, "pr", openError);
 		if (!openError.empty())
 		{
-			fatal("unable to open gpg pipe: %s", openError.c_str());
+			fatal2("unable to open gpg pipe: %s", openError);
 		}
 
 		smatch m;
@@ -378,7 +418,7 @@ bool verifySignature(const Config& config, const string& path)
 				gpgPipe.getLine(result);
 				if (debugging && !gpgPipe.eof())
 				{
-					debug("fetched '%s' from gpg pipe", result.c_str());
+					debug2("fetched '%s' from gpg pipe", result);
 				}
 			} while (!gpgPipe.eof() && (
 						regex_search(result, m, sigIdRegex, regex_constants::match_continuous) ||
@@ -399,14 +439,14 @@ bool verifySignature(const Config& config, const string& path)
 		if (status.empty())
 		{
 			// no info from gpg at all
-			fatal("gpg: '%s': no info received", path.c_str());
+			fatal2("gpg: '%s': no info received", path);
 		}
 
 		// first line ought to be validness indicator
 		static const sregex messageRegex = sregex::compile("(\\w+) (.*)");
 		if (!regex_match(status, m, messageRegex))
 		{
-			fatal("gpg: '%s': invalid status string '%s'", path.c_str(), status.c_str());
+			fatal2("gpg: '%s': invalid status string '%s'", path, status);
 		}
 
 		string messageType = m[1];
@@ -417,12 +457,12 @@ bool verifySignature(const Config& config, const string& path)
 			string furtherInfo = gpgGetLine();
 			if (furtherInfo.empty())
 			{
-				fatal("gpg: '%s': error: unfinished status", path.c_str());
+				fatal2("gpg: '%s': error: unfinished status", path);
 			}
 
 			if (!regex_match(furtherInfo, m, messageRegex))
 			{
-				fatal("gpg: '%s': invalid further info string '%s'", path.c_str(), furtherInfo.c_str());
+				fatal2("gpg: '%s': invalid further info string '%s'", path, furtherInfo);
 			}
 
 			string furtherInfoType = m[1];
@@ -434,25 +474,24 @@ bool verifySignature(const Config& config, const string& path)
 			}
 			else if (furtherInfoType == "EXPSIG")
 			{
-				warn("gpg: '%s': expired signature: %s", path.c_str(), furtherInfoMessage.c_str());
+				warn2("gpg: '%s': expired signature: %s", path, furtherInfoMessage);
 			}
 			else if (furtherInfoType == "EXPKEYSIG")
 			{
-				warn("gpg: '%s': expired key: %s", path.c_str(), furtherInfoMessage.c_str());
+				warn2("gpg: '%s': expired key: %s", path, furtherInfoMessage);
 			}
 			else if (furtherInfoType == "REVKEYSIG")
 			{
-				warn("gpg: '%s': revoked key: %s", path.c_str(), furtherInfoMessage.c_str());
+				warn2("gpg: '%s': revoked key: %s", path, furtherInfoMessage);
 			}
 			else
 			{
-				warn("gpg: '%s': unknown error: %s %s",
-						path.c_str(), furtherInfoType.c_str(), furtherInfoMessage.c_str());
+				warn2("gpg: '%s': unknown error: %s %s", path, furtherInfoType, furtherInfoMessage);
 			}
 		}
 		else if (messageType == "BADSIG")
 		{
-			warn("gpg: '%s': bad signature: %s", path.c_str(), message.c_str());
+			warn2("gpg: '%s': bad signature: %s", path, message);
 		}
 		else if (messageType == "ERRSIG")
 		{
@@ -465,7 +504,7 @@ bool verifySignature(const Config& config, const string& path)
 			{
 				if (!regex_match(detail, m, messageRegex))
 				{
-					fatal("gpg: '%s': invalid detailed info string '%s'", path.c_str(), detail.c_str());
+					fatal2("gpg: '%s': invalid detailed info string '%s'", path, detail);
 				}
 				string detailType = m[1];
 				string detailMessage = m[2];
@@ -477,38 +516,37 @@ bool verifySignature(const Config& config, const string& path)
 					//
 					// NO_PUBKEY D4F5CE00FA0E9B9D
 					//
-					warn("gpg: '%s': public key '%s' not found", path.c_str(), detailMessage.c_str());
+					warn2("gpg: '%s': public key '%s' not found", path, detailMessage);
 				}
 			}
 
 			if (!publicKeyWasNotFound)
 			{
-				warn("gpg: '%s': could not verify signature: %s", path.c_str(), message.c_str());
+				warn2("gpg: '%s': could not verify signature: %s", path, message);
 			}
 		}
 		else if (messageType == "NODATA")
 		{
 			// no signature
-			warn("gpg: '%s': empty signature", path.c_str());
+			warn2("gpg: '%s': empty signature", path);
 		}
 		else if (messageType == "KEYEXPIRED")
 		{
-			warn("gpg: '%s': expired key: %s", path.c_str(), message.c_str());
+			warn2("gpg: '%s': expired key: %s", path, message);
 		}
 		else
 		{
-			warn("gpg: '%s': unknown message received: %s %s",
-					path.c_str(), messageType.c_str(), message.c_str());
+			warn2("gpg: '%s': unknown message received: %s %s", path, messageType, message);
 		}
 	}
 	catch (Exception&)
 	{
-		warn("error while verifying signature for file '%s'", path.c_str());
+		warn2("error while verifying signature for file '%s'", path);
 	}
 
 	if (debugging)
 	{
-		debug("the verify result is %u", (unsigned int)verifyResult);
+		debug2("the verify result is %u", (unsigned int)verifyResult);
 	}
 	return verifyResult;
 }
@@ -517,12 +555,13 @@ shared_ptr< cache::ReleaseInfo > getReleaseInfo(const Config& config, const stri
 {
 	shared_ptr< cache::ReleaseInfo > result(new cache::ReleaseInfo);
 	result->notAutomatic = false; // default
+	result->butAutomaticUpgrades = false; // default
 
 	string openError;
 	File file(path, "r", openError);
 	if (!openError.empty())
 	{
-		fatal("unable to open release file '%s': EEE", path.c_str());
+		fatal2("unable to open release file '%s': %s", path, openError);
 	}
 
 	size_t lineNumber = 1;
@@ -568,6 +607,10 @@ shared_ptr< cache::ReleaseInfo > getReleaseInfo(const Config& config, const stri
 			{
 				result->notAutomatic = true;
 			}
+			else if (fieldName == "ButAutomaticUpgrades")
+			{
+				result->butAutomaticUpgrades = true;
+			}
 			else if (fieldName == "Architectures")
 			{
 				result->architectures = split(' ', fieldValue);
@@ -586,16 +629,16 @@ shared_ptr< cache::ReleaseInfo > getReleaseInfo(const Config& config, const stri
 	}
 	catch (Exception&)
 	{
-		fatal("error parsing release file '%s', line %u", path.c_str(), lineNumber);
+		fatal2("error parsing release file '%s', line %u", path, lineNumber);
 	}
 
 	if (result->vendor.empty())
 	{
-		warn("no vendor specified in release file '%s'", path.c_str());
+		warn2("no vendor specified in release file '%s'", path);
 	}
 	if (result->archive.empty())
 	{
-		warn("no archive specified in release file '%s'", path.c_str());
+		warn2("no archive specified in release file '%s'", path);
 	}
 
 	{ // checking Valid-Until
@@ -617,14 +660,14 @@ shared_ptr< cache::ReleaseInfo > getReleaseInfo(const Config& config, const stri
 				if (mktime(&currentTm) > mktime(&validUntilTm))
 				{
 					bool warnOnly = config.getBool("cupt::cache::release-file-expiration::ignore");
-					(warnOnly ? warn : fatal)("release file '%s' has expired (expiry time '%s')",
-							path.c_str(), result->validUntilDate.c_str());
+					(warnOnly ? warn2< string, string > : fatal2< string, string >)
+							("release file '%s' has expired (expiry time '%s')", path, result->validUntilDate);
 				}
 			}
 			else
 			{
-				warn("unable to parse expiry time '%s' in release file '%s'",
-						result->validUntilDate.c_str(), path.c_str());
+				warn2("unable to parse expiry time '%s' in release file '%s'",
+						result->validUntilDate, path);
 			}
 		}
 	}
